Package: tablet
Type: Package
Title: Tabulate Descriptive Statistics in Multiple Formats
Version: 0.9.1
Authors@R: person(given = "Tim",
                  family = "Bergsma",
                  role = c("aut", "cre"),
                  email = "bergsmat@gmail.com")
Maintainer: Tim Bergsma <bergsmat@gmail.com>
BugReports: https://github.com/bergsmat/tablet/issues
Description: Creates a table of descriptive statistics
 for factor and numeric columns in a data frame. Displays
 these by groups, if any. Highly customizable, with support
 for 'html' and 'pdf' provided by 'kableExtra'. Respects
 original column order, column labels, and factor level order.
 See ?tablet.data.frame and vignettes.
License: GPL-3
Encoding: UTF-8
Imports: dplyr (>= 1.0.2), yamlet (>= 0.10.21), rlang, tidyr,
        kableExtra (>= 0.9.0), spork (>= 0.2.7), magrittr, fs,
        reactable
RoxygenNote: 7.3.3
VignetteBuilder: knitr
Suggests: knitr, rmarkdown, boot, testthat, shiny, shinyFiles, haven,
        yaml, sortable, latexpdf, tinytex, tools, csv, xtable,
        shinyAce, R.utils
NeedsCompilation: no
Packaged: 2026-07-19 03:37:58 UTC; tim.bergsma
Author: Tim Bergsma [aut, cre]
Repository: CRAN
Date/Publication: 2026-07-19 06:00:02 UTC
Built: R 4.6.0; ; 2026-07-19 08:29:01 UTC; unix
