# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,gridtable)
S3method(as_calibrated,data.frame)
S3method(as_gridtable,character)
S3method(as_gridtable,data.frame)
S3method(as_kable,tablet)
S3method(as_tablet,data.frame)
S3method(as_xtable,tablet)
S3method(categoricals,data.frame)
S3method(classifiers,data.frame)
S3method(devalued,observations)
S3method(groupfull,data.frame)
S3method(groupless,data.frame)
S3method(groupwise,data.frame)
S3method(header_rows,tablet)
S3method(headerlist,tablette)
S3method(index,tablette)
S3method(io_csv,tablet)
S3method(io_tablet,character)
S3method(io_tablet,data.frame)
S3method(io_tablet,tablet)
S3method(kbl,calibrated)
S3method(kbl,default)
S3method(kbl,gridtable)
S3method(kbl,tablet)
S3method(numerics,data.frame)
S3method(observations,data.frame)
S3method(print,gridtable)
S3method(recap,knitr_kable)
S3method(splice,data.frame)
S3method(tablet,data.frame)
S3method(tablet,tablette)
S3method(tablette,groupwise)
S3method(tablette,tablet)
S3method(widgets,devalued)
export(absolutizePath)
export(as_calibrated)
export(as_gridtable)
export(as_kable)
export(as_tablet)
export(as_xtable)
export(categoricals)
export(classifiers)
export(devalued)
export(escape_latex)
export(escape_latex.default)
export(escape_latex.latex)
export(groupfull)
export(groupless)
export(groupwise)
export(header_rows)
export(headerlist)
export(index)
export(io_tablet)
export(kbl)
export(mesa)
export(numerics)
export(observations)
export(recap)
export(relativizePath)
export(splice)
export(tablet)
export(tablette)
export(widgets)
importFrom(dplyr,across)
importFrom(dplyr,add_count)
importFrom(dplyr,add_tally)
importFrom(dplyr,all_of)
importFrom(dplyr,anti_join)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_rows)
importFrom(dplyr,case_when)
importFrom(dplyr,distinct)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,full_join)
importFrom(dplyr,group_by)
importFrom(dplyr,group_vars)
importFrom(dplyr,groups)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,slice)
importFrom(dplyr,tally)
importFrom(dplyr,ungroup)
importFrom(fs,is_absolute_path)
importFrom(kableExtra,add_header_above)
importFrom(kableExtra,column_spec)
importFrom(kableExtra,kbl)
importFrom(kableExtra,linebreak)
importFrom(kableExtra,pack_rows)
importFrom(magrittr,"%<>%")
importFrom(magrittr,"%>%")
importFrom(reactable,reactable)
importFrom(reactable,reactableOutput)
importFrom(reactable,renderReactable)
importFrom(rlang,":=")
importFrom(rlang,as_string)
importFrom(rlang,eval_tidy)
importFrom(rlang,f_lhs)
importFrom(rlang,f_rhs)
importFrom(spork,as_html)
importFrom(spork,as_latex)
importFrom(spork,as_spork)
importFrom(tidyr,gather)
importFrom(tidyr,replace_na)
importFrom(tidyr,spread)
importFrom(yamlet,as_dvec)
importFrom(yamlet,io_csv)
importFrom(yamlet,modify)
