% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gridtable.R
\name{as_gridtable.data.frame}
\alias{as_gridtable.data.frame}
\title{Convert a Data Frame to gridtable}
\usage{
\method{as_gridtable}{data.frame}(x, ...)
}
\arguments{
\item{x}{A data frame.}

\item{...}{Currently unused.}
}
\value{
Character scalar with class \code{gridtable} containing a canonical
  Pandoc grid table.
}
\description{
Build a simple Pandoc grid table from a data frame. Column names become the
grid-table header row, each data-frame row becomes one body row, and column
widths are calibrated from the widest formatted value in each column.
}
\seealso{
\code{\link{as_gridtable}}, \code{\link{as_gridtable.character}}

Other gridtable: 
\code{\link{as.data.frame.gridtable}()},
\code{\link{as_calibrated}()},
\code{\link{as_calibrated.data.frame}()},
\code{\link{as_gridtable}()},
\code{\link{as_gridtable.character}()},
\code{\link{kbl}()},
\code{\link{kbl.calibrated}()},
\code{\link{kbl.default}()},
\code{\link{kbl.gridtable}()},
\code{\link{print.gridtable}()}
}
\concept{gridtable}
