% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tablet.R
\name{headerlist}
\alias{headerlist}
\title{Create Header List.}
\usage{
headerlist(x, ...)
}
\arguments{
\item{x}{object}

\item{...}{passed}
}
\value{
see methods
}
\description{
Creates header list.  Generic, with method \code{\link{headerlist.tablette}}.
}
\examples{
example(classifiers)
headerlist(tablette(groupwise(x)))
}
\keyword{internal}
