% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tablet.R
\name{index.tablette}
\alias{index.tablette}
\title{Create Index for Tablet}
\usage{
\method{index}{tablette}(x, ...)
}
\arguments{
\item{x}{tablet}

\item{...}{ignored}
}
\value{
named integer giving row groupings with names as indicated, repeated integer times
}
\description{
Creates index for tablet.
}
\keyword{internal}
