% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gridtable.R
\name{kbl.gridtable}
\alias{kbl.gridtable}
\title{Render a gridtable with kbl}
\usage{
\method{kbl}{gridtable}(x, ...)
}
\arguments{
\item{x}{Character scalar with class \code{gridtable}.}

\item{...}{Additional arguments passed to \code{kbl}.}
}
\value{
A kable object.
}
\description{
Coerce a gridtable to a calibrated data frame with \code{as.data.frame}
before redispatching to \code{kbl}. This lets gridtable-derived column
widths be consumed by \code{kbl.calibrated}.
}
\seealso{
\code{\link{kbl}}, \code{\link{kbl.calibrated}}, \code{\link{as.data.frame.gridtable}}

Other gridtable: 
\code{\link{as.data.frame.gridtable}()},
\code{\link{as_calibrated}()},
\code{\link{as_calibrated.data.frame}()},
\code{\link{as_gridtable}()},
\code{\link{as_gridtable.character}()},
\code{\link{as_gridtable.data.frame}()},
\code{\link{kbl}()},
\code{\link{kbl.calibrated}()},
\code{\link{kbl.default}()},
\code{\link{print.gridtable}()}
}
\concept{gridtable}
